"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var io_ts_1 = require("io-ts");
var io_ts_reporters_1 = require("io-ts-reporters");
var isErrors = function (result) {
    try {
        var keys_1 = ['value', 'context', 'message'];
        return Object.keys(result[0]).every(function (key) { return keys_1.includes(key); });
    }
    catch (e) {
        return false;
    }
};
function default_1(schema) {
    var validateConfigObject = function (data) {
        var result = schema.decode(data);
        if (isErrors(result.value))
            return io_ts_reporters_1.reporter(result);
        else
            return true;
    };
    var loadConfigObject = function (data) {
        var result = schema.decode(data);
        if (isErrors(result.value))
            throw new Error(io_ts_reporters_1.reporter(result).join('\n'));
        else
            return result.value;
    };
    var loadConfigFile = function (filepath) {
        return loadConfigObject(require(filepath));
    };
    return {
        loadConfigObject: loadConfigObject,
        loadConfigFile: loadConfigFile,
        validateConfigObject: validateConfigObject
    };
}
exports.default = default_1;
;
exports.withDefault = function (codec, defaultValue) {
    var name = "default(" + codec.name + ")";
    var isDefaultValid = codec.is(defaultValue);
    return new io_ts_1.Type(name, codec.is, function (i, context) {
        var validation = codec.validate(i, context);
        return validation.isLeft() && isDefaultValid && (i === undefined || i === null) ? io_ts_1.success(defaultValue) : validation;
    }, codec.encode);
};
